package net.wanji.opt.service.signalopt.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.Constants;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.GreenwaveCrossPO;
import net.wanji.databus.dao.entity.GreenwaveHistPO;
import net.wanji.databus.dao.entity.GreenwaveInfoPO;
import net.wanji.databus.dao.entity.GreenwaveRealtimePO;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.databus.po.CrossDataRealtimePO;
import net.wanji.databus.po.CrossDirDataHistPO;
import net.wanji.opt.cache.BaseCrossInfoCache;
import net.wanji.opt.cache.GreenWaveInfoCache;
import net.wanji.opt.common.enums.GreenBeltDirEnum;
import net.wanji.opt.common.enums.GreenWaveInDirEnum;
import net.wanji.opt.dao.mapper.StrategyGreenOptHistMapper;
import net.wanji.opt.po.StrategyGreenOptHistEntity;
import net.wanji.opt.service.signalopt.GreenBeltInfoService;
import net.wanji.opt.vo.*;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.text.DecimalFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @date 2024/12/02 13:42
 */
@Service
@Slf4j
public class GreenBeltServiceImpl implements GreenBeltInfoService {

    @Resource
    private GreenwaveInfoMapper greenwaveInfoMapper;
    @Resource
    private CrossDataHistMapper crossDataHistMapper;
    @Resource
    private StrategyGreenOptHistMapper strategyGreenOptHistMapper;
    @Resource
    private GreenwaveRealtimeMapper greenwaveRealtimeMapper;
    @Resource
    private GreenwaveHistMapper greenwaveHistMapper;
    @Resource
    private GreenwaveCrossMapper greenwaveCrossMapper;
    @Resource
    private CrossDataRealtimeMapper crossDataRealtimeMapper;
    @Resource
    private BaseCrossInfoCache baseCrossInfoCache;
    @Resource
    private CrossDirDataHistMapper crossDirDataHistMapper;


    @Override
    public GreenBeltCrossDetailVO greenBeltCrossDetailList(Integer greenId) throws Exception {
        List<GreenwaveCrossPO> crossPOS = greenwaveCrossMapper.selectByGreenwaveId(greenId);
        List<CrossDataRealtimePO> realtimePOS = crossDataRealtimeMapper.selectAll();
        GreenBeltCrossDetailVO greenBeltCrossDetailVO = new GreenBeltCrossDetailVO();
        LambdaQueryWrapper<GreenwaveRealtimePO> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(GreenwaveRealtimePO::getGreenId, greenId);
        if (!CollectionUtils.isEmpty(crossPOS) && !CollectionUtils.isEmpty(realtimePOS)) {
            List<GreenBeltCrossDetailVO.CrossDetail> detailList = new ArrayList<>();
            for (GreenwaveCrossPO crossPO : crossPOS) {
                String crossId = crossPO.getCrossId();
                GreenBeltCrossDetailVO.CrossDetail crossDetail = new GreenBeltCrossDetailVO.CrossDetail();
                for (CrossDataRealtimePO realtimePO : realtimePOS) {
                    if (StringUtils.equalsIgnoreCase(crossPO.getCrossId(), realtimePO.getCrossId())) {
                        String crossName = baseCrossInfoCache.getCrossName(crossId);
                        crossDetail.setCrossName(crossName);
                        crossDetail.setCrossIndex(realtimePO.getTrafficIndex());
                        detailList.add(crossDetail);
                    }
                }
            }
            List<GreenwaveRealtimePO> greenRealTimes = greenwaveRealtimeMapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(greenRealTimes)) {
                double asDouble = greenRealTimes.stream().mapToDouble(GreenwaveRealtimePO::getTrafficIndex).average().getAsDouble();
                greenBeltCrossDetailVO.setGreenIndex(asDouble);
            }
            greenBeltCrossDetailVO.setDetailList(detailList);
            Map<Integer, GreenwaveInfoPO> greenWaveMap = GreenWaveInfoCache.greenWaveMap;
            GreenwaveInfoPO greenwaveInfoPO = greenWaveMap.get(greenId);
            greenBeltCrossDetailVO.setGreenName(greenwaveInfoPO.getName());
            greenBeltCrossDetailVO.setCrossNum(detailList.size());

        }
        return greenBeltCrossDetailVO;
    }

    @Override
    public Map<String, List<GreenBeltKeyCrossFlowTimeVO>> greenBeltKeyCrossFlowTime(Integer greenId) throws Exception {
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            DecimalFormat df = new DecimalFormat("#0.00");
            List<GreenBeltKeyCrossFlowTimeVO> tempResult = new ArrayList<>();
            List<GreenBeltKeyCrossFlowTimeVO> results = new ArrayList<>();
            LocalDate currentDate = LocalDate.now();
            LocalTime startTime = LocalTime.MIDNIGHT;
            LocalDateTime startOfDay = LocalDateTime.of(currentDate, startTime);

            // 绿波优化数据
            LambdaQueryWrapper<StrategyGreenOptHistEntity> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(StrategyGreenOptHistEntity::getGreenId, greenId);
            queryWrapper.ge(StrategyGreenOptHistEntity::getControlTime, startOfDay);
            List<StrategyGreenOptHistEntity> entities = strategyGreenOptHistMapper.selectList(queryWrapper);
            // 随机路口编号，通过路口编号查询时间轴
            String tempCrossId = "";
            if (!CollectionUtils.isEmpty(entities)) {
                for (StrategyGreenOptHistEntity entity : entities) {
                    GreenBeltKeyCrossFlowTimeVO vo = new GreenBeltKeyCrossFlowTimeVO();
                    String keyCross = entity.getKeyCross();
                    vo.setCrossId(keyCross);
                    vo.setDirName(GreenBeltDirEnum.getDesc(entity.getDir()));
                    Date controlTime = get5MinuteDate(entity.getControlTime());
                    vo.setStartTime(controlTime);
                    String crossGreenDetail = entity.getCrossGreenDetail();
                    Integer cycle = entity.getCycle();
                    List<GreenBeltInfoVO.CrossGreenDetail> crossGreenDetails = mapper.readValue(crossGreenDetail, new TypeReference<List<GreenBeltInfoVO.CrossGreenDetail>>() {
                    });
                    if (!CollectionUtils.isEmpty(crossGreenDetails)) {
                        List<GreenBeltInfoVO.CrossGreenDetail> keyCrossDetail = crossGreenDetails.stream().filter(details -> StringUtils.equalsIgnoreCase(keyCross, details.getCrossId())).collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(keyCrossDetail)) {
                            GreenBeltInfoVO.CrossGreenDetail curKeyCross = keyCrossDetail.get(0);
                            tempCrossId = curKeyCross.getCrossId();
                            Double phaseStartTime = curKeyCross.getPhaseStartTime();
                            Double phaseEndTime = curKeyCross.getPhaseEndTime();
                            Double greenTimeRatio = Double.parseDouble(df.format(phaseEndTime - phaseStartTime / cycle));
                            vo.setGreenTimeRatio(greenTimeRatio);
                        }
                    }
                    tempResult.add(vo);
                }

                // 关键路口历史数据
                LambdaQueryWrapper<CrossDataHistPO> histQuery = new LambdaQueryWrapper<>();
                histQuery.eq(CrossDataHistPO::getCrossId, tempCrossId);
                histQuery.ge(CrossDataHistPO::getStartTime, startOfDay);
                List<CrossDataHistPO> crossDataHistPOS = crossDataHistMapper.selectList(histQuery);
                if (!CollectionUtils.isEmpty(crossDataHistPOS)) {
                    for (CrossDataHistPO crossDataHistPO : crossDataHistPOS) {
                        Date crossStartTime = crossDataHistPO.getStartTime();
                        String format = DateUtil.format(crossStartTime, Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND);
                        Date minuteDate = get5MinuteDate(format);
                        Integer flow = crossDataHistPO.getFlow();
                        GreenBeltKeyCrossFlowTimeVO resultVO = new GreenBeltKeyCrossFlowTimeVO();
                        resultVO.setStartTime(minuteDate);
                        resultVO.setFlow(0);
                        resultVO.setGreenTimeRatio(0.0);
                        resultVO.setCrossId(tempCrossId);
                        resultVO.setDirName("未执行");
                        String crossName = baseCrossInfoCache.getCrossName(tempCrossId);
                        resultVO.setCrossName(crossName);
                        for (GreenBeltKeyCrossFlowTimeVO result : tempResult) {
                            if (StringUtils.equalsIgnoreCase(tempCrossId, result.getCrossId())
                                    && minuteDate.getTime() == result.getStartTime().getTime()) {
                                resultVO.setFlow(flow);
                                resultVO.setGreenTimeRatio(result.getGreenTimeRatio());
                            }
                        }
                        results.add(resultVO);
                    }
                }
            }
            Map<String, List<GreenBeltKeyCrossFlowTimeVO>> map = results.stream().collect(Collectors.groupingBy(GreenBeltKeyCrossFlowTimeVO::getDirName));
            return map;
        } catch (Exception e) {
            log.error("绿波关键路口流量绿信比查询异常：", e);
            throw new RuntimeException(e);
        }
    }


    @Override
    public Map<String, List<GreenBeltSpeedWidthVO>> greenBeltSpeedWidth(Integer greenId) throws Exception {
        try {
            List<GreenBeltSpeedWidthVO> results = new ArrayList<>();
            LocalDate currentDate = LocalDate.now();
            LocalTime startTime = LocalTime.MIDNIGHT;
            LocalDateTime startOfDay = LocalDateTime.of(currentDate, startTime);
            LambdaQueryWrapper<GreenwaveHistPO> greenWrapper = new LambdaQueryWrapper<>();
            greenWrapper.eq(GreenwaveHistPO::getGreenId, greenId);
            greenWrapper.ge(GreenwaveHistPO::getStartTime, startOfDay);
            // 绿波历史数据
            List<GreenwaveHistPO> greenwaveHistPOS = greenwaveHistMapper.selectList(greenWrapper);
            // 绿波优化数据
            LambdaQueryWrapper<StrategyGreenOptHistEntity> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(StrategyGreenOptHistEntity::getGreenId, greenId);
            queryWrapper.ge(StrategyGreenOptHistEntity::getControlTime, startOfDay);
            List<StrategyGreenOptHistEntity> entities = strategyGreenOptHistMapper.selectList(queryWrapper);
            if (!CollectionUtils.isEmpty(greenwaveHistPOS) && !CollectionUtils.isEmpty(entities)) {
                for (GreenwaveHistPO histPO : greenwaveHistPOS) {
                    Integer curGreenId = histPO.getGreenId();
                    Date curStartTime = histPO.getStartTime();
                    String curStartTimeStr = DateUtil.format(curStartTime, Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND);
                    Date startTimeminuteDate = get5MinuteDate(curStartTimeStr);
                    String roadDirection = histPO.getRoadDirection();
                    Double speed = histPO.getSpeed();
                    for (StrategyGreenOptHistEntity entity : entities) {
                        Integer optGreenId = entity.getGreenId();
                        String controlTime = entity.getControlTime();
                        String dir = entity.getDir();
                        Date minuteDate = get5MinuteDate(controlTime);
                        Double greenWidthTime = entity.getGreenWidthTime();
                        if (Objects.equals(curGreenId, optGreenId) && StringUtils.equalsIgnoreCase(dir, roadDirection)
                                && minuteDate.getTime() == startTimeminuteDate.getTime()) {
                            GreenBeltSpeedWidthVO vo = new GreenBeltSpeedWidthVO();
                            vo.setStartTime(curStartTime);
                            vo.setSpeed(speed);
                            vo.setWidth(greenWidthTime);
                            vo.setDirName(GreenBeltDirEnum.getDesc(dir));
                            results.add(vo);
                        }

                    }
                }
            }
            Map<String, List<GreenBeltSpeedWidthVO>> map = results.stream().collect(Collectors.groupingBy(GreenBeltSpeedWidthVO::getDirName));
            return map;
        } catch (Exception e) {
            log.error("绿波带宽曲线异常：", e);
            throw new RuntimeException(e);
        }
    }

    /**
     * 将下发时间按五分钟取整
     *
     * @param controlTime
     * @return
     */
    public static Date get5MinuteDate(String controlTime) {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        LocalDateTime dateTime = LocalDateTime.parse(controlTime, formatter);
        // 获取当前分钟
        int minute = dateTime.getMinute();
        // 计算需要补充的分钟数（如果 minute % 5 != 0）
        int minutesToAdd = (5 - minute % 5) % 5;
        // 添加这些分钟并返回新的时间
        LocalDateTime localDateTime = dateTime.plusMinutes(minutesToAdd).withSecond(0).withNano(0);
        ZoneId zoneId = ZoneId.systemDefault();
        Date minuteDate = Date.from(localDateTime.atZone(zoneId).toInstant());
        return minuteDate;
    }

    @Override
    public List<GreenBeltFlowStopTimeVO> greenBeltCrossDetailHist(Integer greenId) throws Exception {
        try {
            List<String> crossIds = greenwaveInfoMapper.selectCrossIdsById(greenId);
            GreenwaveInfoPO greenwaveInfoPO = greenwaveInfoMapper.selectById(greenId);
            long currentTimeMillis = System.currentTimeMillis();
            int startSecond = ((int) (currentTimeMillis / 1000)) - 3600;
            int endSecond = (int) (currentTimeMillis / 1000);
            List<GreenBeltFlowStopTimeVO> results = new ArrayList<>();
            if (Objects.nonNull(greenwaveInfoPO)) {
                String greenDir = greenwaveInfoPO.getGreenDir();
                String[] split = greenDir.split(",");
                List<Integer> dirList = new ArrayList<>(split.length);
                for (String s : split) {
                    dirList.add(Integer.valueOf(s));
                }
                List<CrossDirDataHistPO> dirDataHistPOS = crossDirDataHistMapper.selectByCrossIdsDirsAndTimestamp(crossIds, dirList, startSecond, endSecond);
                if (!CollectionUtils.isEmpty(dirDataHistPOS)) {
                    Map<Date, List<CrossDirDataHistPO>> dateMap = dirDataHistPOS.stream().collect(Collectors.groupingBy(CrossDirDataHistPO::getStartTime));
                    for (Map.Entry<Date, List<CrossDirDataHistPO>> entry : dateMap.entrySet()) {
                        Date date = entry.getKey();
                        List<CrossDirDataHistPO> value = entry.getValue();
                        GreenBeltFlowStopTimeVO greenBeltFlowStopTimeVO = new GreenBeltFlowStopTimeVO();
                        greenBeltFlowStopTimeVO.setStartTime(date);
                        List<GreenBeltFlowStopTimeVO.FlowStopTimeDetail> detailList = new ArrayList<>();
                        if (!CollectionUtils.isEmpty(value)) {
                            for (CrossDirDataHistPO crossDirDataHistPO : value) {
                                GreenBeltFlowStopTimeVO.FlowStopTimeDetail detail = new GreenBeltFlowStopTimeVO.FlowStopTimeDetail();
                                Integer dirType = crossDirDataHistPO.getDirType();
                                detail.setGreenDir(GreenWaveInDirEnum.getDesc(dirType));
                                detail.setCrossId(crossDirDataHistPO.getCrossId());
                                detail.setFlow(crossDirDataHistPO.getFlow());
                                detail.setStopTimes(crossDirDataHistPO.getStopTimes());
                                detailList.add(detail);
                            }
                        }
                        greenBeltFlowStopTimeVO.setDetailList(detailList);
                        results.add(greenBeltFlowStopTimeVO);
                    }
                }
                Collections.sort(results, Comparator.comparing(GreenBeltFlowStopTimeVO::getStartTime));
                return results;
            }
            return Collections.EMPTY_LIST;
        } catch (Exception e) {
            log.error("绿波协调方向路口流量停车次数查询异常：", e);
            throw new RuntimeException(e);
        }
    }
}
